@@ -14,7 +14,11 @@
 namespace OCA\Bookmarks\AppInfo;
 
 use OCA\Bookmarks\Controller\Lib\Bookmarks;
+use OCA\Bookmarks\Controller\Lib\Previews\DefaultPreviewService;
+use OCA\Bookmarks\Controller\Lib\Previews\ScreenlyPreviewService;
+use OCA\Bookmarks\Controller\Lib\Previews\FaviconPreviewService;
 use \OCP\AppFramework\App;
+use OCP\AppFramework\Utility\ITimeFactory;
 use \OCP\IContainer;
 use \OCA\Bookmarks\Controller\WebViewController;
 use OCA\Bookmarks\Controller\Rest\TagsController;
@@ -22,11 +26,11 @@ use OCA\Bookmarks\Controller\Rest\BookmarkController;
 use OCA\Bookmarks\Controller\Rest\InternalTagsController;
 use OCA\Bookmarks\Controller\Rest\InternalBookmarkController;
 use OCA\Bookmarks\Controller\Rest\PublicController;
+use OCA\Bookmarks\Controller\Rest\SettingsController;
 use OCP\IUser;
 
 class Application extends App {
-
-	public function __construct(array $urlParams = array()) {
+	public function __construct(array $urlParams = []) {
 		parent::__construct('bookmarks', $urlParams);
 
 		$container = $this->getContainer();
@@ -35,7 +39,7 @@ class Application extends App {
 		 * Controllers
 		 * @param IContainer $c The Container instance that handles the request
 		 */
-		$container->registerService('WebViewController', function($c) {
+		$container->registerService('WebViewController', function ($c) {
 			/** @var IUser|null $user */
 			$user = $c->query('ServerContainer')->getUserSession()->getUser();
 			$uid = is_null($user) ? null : $user->getUID();
@@ -51,7 +55,7 @@ class Application extends App {
 			);
 		});
 
-		$container->registerService('BookmarkController', function($c) {
+		$container->registerService('BookmarkController', function ($c) {
 			/** @var IContainer $c */
 			$user = $c->query('ServerContainer')->getUserSession()->getUser();
 			$uid = is_null($user) ? null : $user->getUID();
@@ -65,11 +69,12 @@ class Application extends App {
 				$c->query('ServerContainer')->getUserManager()
 			);
 		});
-		
-		$container->registerService('InternalBookmarkController', function($c) {
+
+		$container->registerService('InternalBookmarkController', function ($c) {
 			/** @var IContainer $c */
 			$user = $c->query('ServerContainer')->getUserSession()->getUser();
 			$uid = is_null($user) ? null : $user->getUID();
+
 			return new InternalBookmarkController(
 				$c->query('AppName'),
 				$c->query('Request'),
@@ -77,11 +82,15 @@ class Application extends App {
 				$c->query('ServerContainer')->getDatabaseConnection(),
 				$c->query('ServerContainer')->getL10NFactory()->get('bookmarks'),
 				$c->query('ServerContainer')->query(Bookmarks::class),
-				$c->query('ServerContainer')->getUserManager()
+				$c->query('ServerContainer')->getUserManager(),
+				$c->query('ServerContainer')->query(DefaultPreviewService::class),
+				$c->query('ServerContainer')->query(FaviconPreviewService::class),
+				$c->query('ServerContainer')->query(ScreenlyPreviewService::class),
+				$c->query('ServerContainer')->query(ITimeFactory::class)
 			);
 		});
 
-		$container->registerService('TagsController', function($c) {
+		$container->registerService('TagsController', function ($c) {
 			/** @var IContainer $c */
 			$user = $c->query('ServerContainer')->getUserSession()->getUser();
 			$uid = is_null($user) ? null : $user->getUID();
@@ -92,8 +101,8 @@ class Application extends App {
 				$c->query('ServerContainer')->query(Bookmarks::class)
 			);
 		});
-		
-		$container->registerService('InternalTagsController', function($c) {
+
+		$container->registerService('InternalTagsController', function ($c) {
 			/** @var IContainer $c */
 			$user = $c->query('ServerContainer')->getUserSession()->getUser();
 			$uid = is_null($user) ? null : $user->getUID();
@@ -105,7 +114,7 @@ class Application extends App {
 			);
 		});
 
-		$container->registerService('PublicController', function($c) {
+		$container->registerService('PublicController', function ($c) {
 			/** @var IContainer $c */
 			$user = $c->query('ServerContainer')->getUserSession()->getUser();
 			$uid = is_null($user) ? null : $user->getUID();
@@ -117,6 +126,26 @@ class Application extends App {
 				$c->query('ServerContainer')->getUserManager()
 			);
 		});
-	}
 
+		$container->registerService('SettingsController', function ($c) {
+			/** @var IContainer $c */
+			$user = $c->query('ServerContainer')->getUserSession()->getUser();
+			$uid = is_null($user) ? null : $user->getUID();
+			return new SettingsController(
+				$c->query('AppName'),
+				$c->query('Request'),
+				$uid,
+				$c->query('ServerContainer')->getConfig()
+			);
+		});
+
+		$container->registerService('RecreateAllBookmarks', function ($c) {
+			/** @var IContainer $c*/
+			return new RecreateAllBookmarks(
+					$c->query('ServerContainer')->getDb(),
+					$c->query('ServerContainer')->query(Bookmarks::class),
+					$c->query('ServerContainer')->getConfig()
+				);
+		});
+	}
 }
