@@ -2,23 +2,18 @@ OC.L10N.register(
     "bookmarks",
     {
     "Bookmarks" : "ブックマーク",
+    "Failed to import one bookmark, because: " : "ブックマークのインポートに失敗しました。原因:",
+    "Saved" : "保存しました",
+    "An error occurred while trying to save the bookmark." : "ブックマークの保存を試している際にエラーが発生しました。",
     "Error" : "エラー",
-    "Tags" : "タグ",
-    "Filter by tag" : "タグによるフィルター",
+    "Uploading..." : "アップロード中...",
     "Import error" : "インポート エラー",
     "Import completed successfully." : "インポートに成功しました",
-    "Uploading..." : "アップロード中...",
     "Add a bookmark" : "ブックマークへ追加",
-    "This URL is already bookmarked! Overwrite?" : "このURLは既にブックマークされています! 上書きしますか?",
+    "This URL is already bookmarked! Overwrite?" : "このURLはすでにブックマークされています! 上書きしますか?",
     "The title of the page" : "ページのタイトル",
     "The address of the page" : "ページのアドレス",
     "Description of the page" : "ページの説明",
-    "Save" : "保存",
-    "Delete" : "削除",
-    "Edit" : "編集",
-    "Cancel" : "キャンセル",
-    "Export & Import" : "エクスポート & インポート",
-    "Export" : "エクスポート",
-    "Import" : "インポート"
+    "Save" : "保存"
 },
 "nplurals=1; plural=0;");
